
-- the system for powering up the mario
-- from Super Mario Bros. released for
-- the Nintendo Entertainment System

sfxinfo[freeslot("sfx_smbpup")].caption = "Powering up"
sfxinfo[freeslot("sfx_smbpdw")].caption = "Hurt"
sfxinfo[freeslot("sfx_smbbjp")].caption = "Jump"
sfxinfo[freeslot("sfx_smbfba")].caption = "Fireball"

freeslot("SPR_MBFB", "S_SMB_FIREBALL", "S_SMB_FIREBALLDEAD")

states[S_SMB_FIREBALL] = {
    sprite = SPR_MBFB,
    frame = A|FF_ANIMATE,
    tics = -1,
    var1 = D,
	var2 = 3
}

states[S_SMB_FIREBALLDEAD] = {
    sprite = SPR_MBFB,
    frame = E|FF_ANIMATE,
    tics = (G-E+1)*2,
    var1 = (G-E),
	var2 = 2
}

freeslot("S_SMB_GROW", "S_SMB_CROUCHED")

states[S_SMB_GROW] = {
    sprite = SPR_PLAY,
    frame = SPR2_FLY_|FF_ANIMATE,
    tics = -1,
    var1 = D,
	var2 = 3
}

states[S_SMB_CROUCHED] = {
    sprite = SPR_PLAY,
    frame = SPR2_LAND,
    tics = -1
}

addHook("PlayerSpawn", function(p)
	if not (p.mo and p.mo.valid)
	or p.mo.skin ~= "smbmario" then return end
	
	p.smbfblimit = 0
	if p.smbpowerup then
		p.mo.sprite2 = $1|FF_SPR2SUPER
	end
end)

addHook("PlayerThink", function(p)
	if not (p.mo and p.mo.valid)
	or p.mo.skin ~= "smbmario" then
		if p.smbpowerup ~= nil
			p.smbpowerup = nil
			p.mo.eflags = $ & ~(MFE_FORCESUPER|MFE_FORCENOSUPER)
		end
		return
	end
	
	p.smbpowerup = max(min(2, ($ ~= nil and $ or 0)), 0)
	if p.smbpowerup == nil
	or p.playerstate ~= PST_LIVE
		p.smbpowerup = 0
	end
	
	if p.smbfblimit == nil
		p.smbfblimit = 0
	end
	
	if p.smblastpup ~= p.smbpowerup
		if p.smblastpup == 2
			p.mo.color = p.skincolor
		end
		p.smblastpup = p.smbpowerup
	end
	
	if p.smbpowerup ~= 0
		p.mo.eflags = ($1|MFE_FORCESUPER) & ~MFE_FORCENOSUPER
		
		if S_SoundPlaying(p.mo, skins[p.skin].soundsid[SKSJUMP])
			S_StopSoundByID(p.mo, skins[p.skin].soundsid[SKSJUMP])
			S_StartSound(p.mo, sfx_smbbjp)
		end
		
		local crouchbtn = (p.mo.flags2 & MF2_TWOD) and (p.smbfmove and p.smbfmove < 0) or (p.smbbtns and p.smbbtns.crouch and (p.cmd.buttons & p.smbbtns.crouch) and true or false)
		if crouchbtn
		and P_IsObjectOnGround(p.mo)
		and p.playerstate ~= PST_DEAD
		and not P_PlayerInPain(p)
		and (not (p.mo.eflags & MFE_UNDERWATER) or P_IsObjectOnGround(p.mo))
		and not p.exiting
		and not (p.pflags & PF_STASIS)
			p.smbcrouched = true
		end
		
		if p.smbcrouched
			if not crouchbtn
			and P_IsObjectOnGround(p.mo)
			or p.playerstate == PST_DEAD
			or P_PlayerInPain(p)
			or p.smbgetpowerup
			or ((p.mo.eflags & MFE_UNDERWATER) and not P_IsObjectOnGround(p.mo))
			or p.exiting
			or (p.pflags & PF_STASIS)
				if p.playerstate ~= PST_DEAD
				and not P_PlayerInPain(p)
					p.mo.state = S_PLAY_STND
				end
				p.smbcrouched = false
				return
			end
			
			if P_IsObjectOnGround(p.mo)
				p.pflags = $1|PF_STASIS
			end
			if p.mo.state ~= S_SMB_CROUCHED
				p.mo.state = S_SMB_CROUCHED
			end
		end
		
		if p.smbpowerup == 2
			if p.smbbtns
			and p.smbbtns.run
			and (p.cmd.buttons & p.smbbtns.run)
			and not (p.lastbuttons & p.smbbtns.run)
			and p.smbfblimit < 2
			and not p.exiting
			and not (p.pflags & PF_STASIS)
				S_StartSound(p.mo, sfx_smbfba)
				local fb = P_SPMAngle(p.mo, MT_FIREBALL, p.mo.angle, 0)
				if (fb and fb.valid)
					fb.state = S_SMB_FIREBALL
					fb.spritexscale = $*2
					fb.spriteyscale = $*2
					fb.smbfireball = true
					p.smbfblimit = $+1
				end
			end
			
			if p.mo.color == p.skincolor
				local color = SKINCOLOR_SMBFIRE
				if G_GametypeHasTeams()
					if p.ctfteam == 1
						color = SKINCOLOR_APRICOT
					elseif p.ctfteam == 2
						color = SKINCOLOR_TURQUOISE
					end
				end
				p.mo.color = color
			end
		end
	else
		p.mo.eflags = ($1|MFE_FORCENOSUPER) & ~MFE_FORCESUPER
		p.smbcrouched = false
	end
	
	if p.powers[pw_shield]
		P_RemoveShield(p)
		if p.smbpowerup < 2
			p.smbgetpowerup = true
		end
	end
	
	if p.smbgetpowerup
	and p.smbgetpoweruptime ~= nil
		local losepup = p.smblosepowerup and true or false
		if p.smbgetpoweruptime == 0
			S_StartSound(p.mo, losepup and sfx_smbpdw or sfx_smbpup, p)
		end
		
		if p.playerstate == PST_DEAD
			p.smbgetpowerup = false
			return
		end
		
		p.mo.momx = 0
		p.mo.momy = 0
		p.mo.momz = 0
		p.pflags = $1|PF_FULLSTASIS
		if p.mo.state ~= S_SMB_GROW
		and (p.smbpowerup+1 < 2 or losepup)
			p.mo.state = S_SMB_GROW
		elseif p.smbpowerup+1 >= 2
		and not losepup
			if p.mo.state ~= p.smbpupmom.s
				p.mo.state = p.smbpupmom.s
			end
			p.mo.frame = p.smbpupmom.f
			p.mo.tics = p.smbpupmom.t[1]
			p.mo.anim_duration = p.smbpupmom.t[2]
		end
		
		if losepup
			p.mo.sprite2 = $1|FF_SPR2SUPER
		elseif p.smbpowerup+1 < 2
			p.mo.sprite2 = $ & ~FF_SPR2SUPER
		end
		
		if p.smbgetpoweruptime >= TICRATE
			p.pflags = $ & ~PF_FULLSTASIS
			p.mo.momx = p.smbpupmom.x
			p.mo.momy = p.smbpupmom.y
			p.mo.momz = p.smbpupmom.z
			p.mo.state = p.smbpupmom.s
			P_MovePlayer(p)
			local pupnum = min(p.smbpowerup+1, 2)
			if losepup then pupnum = 0 end
			p.smbpowerup = pupnum
			p.smbgetpowerup = false
			return
		end
		p.smbgetpoweruptime = $+1
	else
		p.smbgetpoweruptime = 0
		p.smblosepowerup = false
		p.smbpupmom = {x = p.mo.momx, y = p.mo.momy, z = p.mo.momz, s = p.mo.state, f = p.mo.frame, t = {p.mo.tics, p.mo.anim_duration}}
		if p.smbpupmom.s == S_SMB_CROUCHED
			if P_IsObjectOnGround(p.mo)
				p.smbpupmom.s = S_PLAY_STND
			else
				p.smbpupmom.s = S_PLAY_JUMP
			end
		end
	end
	
	/*if (p.cmd.buttons & BT_CUSTOM2)
		p.smbgetpowerup = true
	end*/
end)

addHook("PlayerHeight", function(p)
	if not (p.mo and p.mo.valid)
	or p.mo.skin ~= "smbmario"
	or not p.smbpowerup
	or p.smbcrouched
	or (p.pflags & PF_SPINNING) then return end
	
	return P_GetPlayerHeight(p)*2
end)

addHook("MobjDamage", function(pmo, inf, src)
	if not (pmo and pmo.valid)
	or pmo.skin ~= "smbmario" then return end
	local p = pmo.player
	
	if p.smbpowerup
		p.smbgetpowerup = true
		p.smblosepowerup = true
	else
		P_KillMobj(pmo, inf, src)
	end
	p.powers[pw_flashing] = 2*TICRATE
	return true
end, MT_PLAYER)

addHook("ShouldDamage", function(pmo, _, _, _, dmg)
	if not (pmo and pmo.valid)
	or pmo.skin ~= "smbmario"
	or not pmo.player.smbgetpowerup
	or (dmg & DMG_DEATHMASK) then return end
	
	return false
end, MT_PLAYER)